#include <string.h>
#include "spec-console.h"

#define SPEC_CONSOLE_NAME "SpecConsole_name"
#define SPEC_CONSOLE_SERVER "SpecConsole_server"
#define SPEC_CONSOLE_TERMINAL "SpecConsole_terminal"

extern void
spec_console_free(SpecConsole *sc) {
    g_assert(sc);
    if (sc->name) {
	g_free(sc->name);
    }
    if (sc->server) {
	g_free(sc->server);
    }
    if (sc->terminal) {
	terminal_free(sc->terminal);
    }
    g_free(sc);
}

extern SpecConsole *
spec_console_new(const gchar *name,
	const gchar *server,
	const gchar *device,
	guint speed,
	const gchar *bits) {
    SpecConsole *sc = g_new0(struct _SpecConsole, 1);

    g_assert(name);
    g_assert(server);
    g_assert(device);
    g_assert(bits);
    sc->name = g_strdup(name);
    sc->server = g_strdup(server);
    if (!strcmp(bits, "7E2")) {
	sc->terminal = terminal_new(device, speed, speed, 7, PARITY_EVEN, 2);
    } else {
	sc->terminal = terminal_new(device, speed, speed, 8, PARITY_NONE, 1);
    }
    return sc;
}

extern MessageArg *
spec_console_message_arg(const gchar *name,
	const SpecConsole *sc) {
    MessageArg *a;

    g_assert(name);
    g_assert(sc);
    a = message_arg_list(name, NULL);
    a->list = g_list_prepend(a->list,
	    message_arg_string(SPEC_CONSOLE_NAME, sc->name?sc->name:""));
    a->list = g_list_prepend(a->list,
	    message_arg_string(SPEC_CONSOLE_SERVER, sc->server?sc->server:""));
    a->list = g_list_prepend(a->list,
	    terminal_message_arg(SPEC_CONSOLE_TERMINAL, sc->terminal));
    return a;
}

extern SpecConsole *
spec_console_new_from_arg(const MessageArg *arg) {
    MessageArg *name;
    MessageArg *server;
    MessageArg *terminal;
    SpecConsole *sc;
    Terminal *t;

    g_assert(arg);
    if (arg->type != MESSAGE_ARG_LIST) return NULL;
    if (message_arg_list_explode(arg,
		SPEC_CONSOLE_NAME, MESSAGE_ARG_STRING, &name,
		SPEC_CONSOLE_SERVER, MESSAGE_ARG_STRING, &server,
		SPEC_CONSOLE_TERMINAL, MESSAGE_ARG_LIST, &terminal,
		NULL, 0, NULL) == FALSE) {
	return NULL;
    }
    t = terminal_new_from_arg(terminal);
    if (t == NULL) {
	return NULL;
    }
    sc = g_new0(struct _SpecConsole, 1);
    sc->name = g_strdup(name->string->str);
    sc->server = g_strdup(server->string->str);
    sc->terminal = t;
    return sc;
}
