#include <string.h>
#include "svr.h"
#include "console.h"
#include "svr-message.h"
#include "terminal.h"

typedef struct _Console {
    Terminal *t;
    GIOChannel *io;
    MplexChannel chan;
    Server *server;
} Console;

static void
console_tree_free(gpointer data) {
    Console *c = data;

    terminal_free(c->t);
    g_io_channel_unref(c->io);
    /* FIXME: do something with the mplex channel? */
    g_free(c);
}

static void
console_open(MessageQueue q,
	const gchar *name,
	MessageID id,
	const MessageArg *arg,
	gpointer data) {
    Server *server = data;
    Terminal *terminal = terminal_new_from_arg(arg);
    MessageArg *a;
    GIOChannel *io;
    GError *e = NULL;
    MplexChannel chan;

    g_assert(server);
    g_assert(terminal);

    g_printerr("attempting to open console %s\n", terminal->device);
    io = terminal_open_io_channel(terminal, &e);
    if (io == NULL) {
	g_printerr(_("failed to open console %s\n"), terminal->device);
	a = message_arg_error(1, "%s", e->message);
	g_clear_error(&e);
	if (message_respond(q, id, a, &e) == FALSE) {
	    g_printerr(_("failed to send reponse (%s)\n"), e->message);
	    g_clear_error(&e);
	}
	message_arg_free(a);
	return;
    }
    if (mplex_connect(server->mplex, 0, &chan, io, &e) == FALSE) {
	g_printerr(_("failed to connect console to mplex (%s)\n"), e->message);
	g_clear_error(&e);
	return;
    }
    /* FIXME: add to consoles tree */
    a = message_arg_uint16("channel", chan);
    if (message_respond(q, id, a, &e) == FALSE) {
	g_printerr(_("failed to send reponse (%s)\n"), e->message);
	g_clear_error(&e);
    }
}

extern void
console_register_message_handlers(Server *server) {
    GError *e = NULL;

    g_assert(server);

    if (server->msg_q == NULL) {
	return;
    }
    if (message_handler_register(server->msg_q,
		SERVER_MESSAGE_OPEN_CONSOLE, console_open,
		server, &e) == FALSE) {
	g_printerr(_("could not register handler %s (%s)\n"),
		SERVER_MESSAGE_OPEN_CONSOLE, e->message);
    }
}

extern GTree *
console_tree_init(void) {
    return g_tree_new_full((GCompareDataFunc)strcmp, NULL,
	    g_free, console_tree_free);
}
